/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.nodeformatter;

import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbException;

import javax.servlet.jsp.PageContext;


/**
 * This utility class provides commonly used functionality to the classes in this package.
 *
 * @author Pops, TCSDEVELOPER
 * @version 2.0
 * @since 2.0
 */
class NodeFormatterUtils {
    /**
     * This is a private constructor to avoid instantiation.
     */
    private NodeFormatterUtils() {
        // this constructor is intentionally empty
    }

    /**
     * This method retrieves the attribute with the given name from the page context. After retrieval it checks whether
     * the retrieved value is of the expected type and throws a BreadCrumbException if the type does not match the
     * expected type. In case <tt>null</tt> has been retrieved from the page context, a BreadCrumbException will be
     * thrown.
     *
     * @param pageContext   the page context to retrieve the value from
     * @param attributeName the name of the attribute whose value to retrieve from the page context
     * @param expectedType  the expected type of the value to be retrieved, cause a BreadCrumbException in case
     *                      mismatches with actual value retrieved
     *
     * @return the non-<tt>null</tt>value retrieved, which is assured to be of the expected type
     *
     * @throws com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbException
     *                                  in case the type of the value retrieved does not match the expected type or the
     *                                  value retrieved was <tt>null</tt>
     * @throws IllegalArgumentException in case any arg is <tt>null</tt> or attributeName is an empty (trim'd) string
     */
    static Object getRequiredPageContextAttribute(final PageContext pageContext, final String attributeName,
                                                  final Class expectedType)
        throws BreadCrumbException {
        //args checked in getOptionalPageContextAttribute
        final Object nodeObj = getOptionalPageContextAttribute(pageContext, attributeName, expectedType);
        if (nodeObj == null) {
            throw new BreadCrumbException("Expected the context attribute named [" + attributeName + "] to contain a ["
                + expectedType.getName() + "], but it contained a null value.");
        }
        return nodeObj;
    }

    /**
     * This method retrieves the attribute with the given name from the page context. After retrieval it checks whether
     * the retrieved value is of the expected type and throws a BreadCrumbException if the type does not match the
     * expected type. In case <tt>null</tt> has been retrieved from the page context, <tt>null</tt> is returned.
     *
     * @param pageContext   the page context to retrieve the value from
     * @param attributeName the name of the attribute whose value to retrieve from the page context
     * @param expectedType  the expected type of the value to be retrieved, cause a BreadCrumbException in case
     *                      mismatches with actual value retrieved
     *
     * @return the value retrieved, which is assured to be of the expected type, but may be <tt>null</tt> in case
     *         <tt>null</tt> has been retried from the page context
     *
     * @throws com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbException
     *                                  in case the type of the value retrieved does not match the expected type
     * @throws IllegalArgumentException in case any arg is <tt>null</tt> or attributeName is an empty (trim'd) string
     */
    static Object getOptionalPageContextAttribute(final PageContext pageContext, final String attributeName,
                                                  final Class expectedType)
        throws BreadCrumbException {
        if (pageContext == null) {
            throw new IllegalArgumentException("The parameter named [pageContext] was null.");
        }
        if (attributeName == null) {
            throw new IllegalArgumentException("The parameter named [attributeName] was null.");
        }
        if (attributeName.trim().length() == 0) {
            throw new IllegalArgumentException("The parameter named [attributeName] was an empty String.");
        }
        if (expectedType == null) {
            throw new IllegalArgumentException("The parameter named [expectedType] was null.");
        }

        final Object nodeObj = pageContext.getAttribute(attributeName);
        if (nodeObj != null && !expectedType.isAssignableFrom(nodeObj.getClass())) {
            throw new BreadCrumbException("Expected the context attribute named [" + attributeName + "] to contain a ["
                + expectedType.getName() + "], but it contained a [" + nodeObj.getClass().getName() + "].");
        }
        return nodeObj;
    }
}
