/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;


/**
 * This is a utility class that contains common utility methods used by classes in this package.
 *
 * @author TCSDEVELOPER
 * @version 2.0
 * @since 2.0
 */
class DiscoveryUtils {
    /**
     * This is a NodeMatcherListComparator instance used in list sorting in {@link #getFirstPath(java.util.List)}.
     */
    private static final NodeMatcherListComparator NODE_MATCHER_LIST_COMPARATOR = new NodeMatcherListComparator();

    /**
     * This is a private constructor to avoid initialization of this utility class.
     */
    private DiscoveryUtils() {
        // This constructor is intentionally empty
    }

    /**
     * This method creates a reverse map from the given site map. This means given a map which maps nodes to a list of
     * their child nodes, it returns a map that maps nodes to a list of their parent nodes, were nodes are instances of
     * {@link NodeMatcher} and the lists are {@link List}s containing elements of type {@link NodeMatcher}.
     *
     * @param siteMap the site map to create the reverse map from
     *
     * @return the reverse map for the given site map
     *
     * @throws IllegalArgumentException in case the given map is <tt>null</tt> or is not a valid siteMap
     */
    static Map createReverseMap(final Map siteMap) {
        if (siteMap == null) {
            throw new IllegalArgumentException("The parameter named [siteMap] was null.");
        }

        final Map ret = new HashMap(siteMap.size());

        final Set set = siteMap.entrySet();
        for (Iterator iterator = set.iterator(); iterator.hasNext();) {
            final Map.Entry entry = (Map.Entry) iterator.next();
            final Object key = entry.getKey();
            if (!(key instanceof NodeMatcher)) {
                throw new IllegalArgumentException(
                    "The given site map contained a key that was not of type NodeMatcher, but was ["
                        + (key == null ? key : key.getClass().getName()) + "].");
            }
            final Object value = entry.getValue();
            if (value != null) {
                if (!(value instanceof List)) {
                    throw new IllegalArgumentException(
                        "The given site map contained a value that was not of type List, but was ["
                            + value.getClass().getName() + "].");
                }
                for (Iterator iterator1 = ((List) value).iterator(); iterator1.hasNext();) {
                    final Object listItem = iterator1.next();
                    if (!(listItem instanceof NodeMatcher)) {
                        throw new IllegalArgumentException(
                            "The given site map contained a child list item that was not of type NodeMatcher, but was ["
                                + (listItem == null ? listItem : listItem.getClass().getName()) + "].");
                    }
                    List parentsOfCurrentChild = (List) ret.get(listItem);
                    if (parentsOfCurrentChild == null) {
                        parentsOfCurrentChild = new ArrayList();
                        ret.put(listItem, parentsOfCurrentChild);
                    }
                    if (!parentsOfCurrentChild.contains(key)) {
                        parentsOfCurrentChild.add(key);
                    }
                }
            }
        }
        return ret;
    }

    /**
     * This method selects the shortest paths to root for a given node. That is given a list of nodes and a reverse map
     * (as created by {@link #createReverseMap(java.util.Map)}) it follows all of these nodes all possible paths to
     * root and returns the path(s) that are the shortest paths from any of the given nodes to a root node.
     *
     * @param candidates a List of NodeMatcher instances that are the start points for the paths from which to determine
     *                   the shortest one
     * @param reverseMap a reverse map, as created by {@link #createReverseMap(java.util.Map)}
     *
     * @return a {@link java.util.List} of {@link List}s that contains {@link NodeMatcher} instances, where all lists in
     *         the outer list are paths of equal length ordered from root to current
     *
     * @throws IllegalArgumentException in case any arg is <tt>null</tt> or the candidates list is empty, contains
     *                                  objects not of type NodeMatcher or the map contains values that are not {@link
     *                                  java.util.List}s of {@link NodeMatcher}s
     */
    static List findShortestPaths(final List candidates, final Map reverseMap) {
        if (candidates == null) {
            throw new IllegalArgumentException("The parameter named [candidates] was null.");
        }
        if (candidates.isEmpty()) {
            throw new IllegalArgumentException("The parameter named [candidates] was an empty list.");
        }
        if (reverseMap == null) {
            throw new IllegalArgumentException("The parameter named [reverseMap] was null.");
        }

        // the list of path candidates to be examined
        // whether they are paths to root already
        //
        // - the value of this variable is a newly calculated list
        // upon each iteration of the while-loop
        List candidatePaths = new ArrayList();

        // the initial candidates list are paths containing
        // the matching matchers as end points
        for (Iterator iterator = candidates.iterator(); iterator.hasNext();) {
            final Object n = iterator.next();
            if (!(n instanceof NodeMatcher)) {
                throw new IllegalArgumentException("Expected the given candidates list to only contain NodeMatchers.");
            }
            final NodeMatcher nodeMatcher = (NodeMatcher) n;
            final List path = new ArrayList();
            path.add(nodeMatcher);
            candidatePaths.add(path);
        }

        // the list containing any paths to be found to have a root node
        // in the current iteration cycle of the while-loop,
        //
        // when this list becomes non-empty, the search is ended after that
        // cycle completes (cycle completion is needed
        // because this will find ALL paths of shortest length instead of only the first one)
        final List shorthestPaths = new ArrayList();

        // BFS: as long as we do not have a shortest path we examine
        // all remaining paths and their parents
        // as alternatives are eliminated by the loop
        // detection, we may end up with an empty
        // candidates list, in which case we also terminate
        while (shorthestPaths.isEmpty() && !candidatePaths.isEmpty()) {

            // first we check whether any of the candidates is a path to root already,
            // in case it is, we add id to the list of found shortest paths
            for (Iterator iterator = candidatePaths.iterator(); iterator.hasNext();) {
                final List currentPath = (List) iterator.next();
                final NodeMatcher current = (NodeMatcher) currentPath.get(0);
                final Object par = reverseMap.get(current);
                if (par != null && !(par instanceof List)) {
                    throw new IllegalArgumentException(
                        "the given map was expected to only contain values of type List.");
                }
                final List parents = (List) par;
                if (parents == null || parents.isEmpty()) {
                    //we have found a root node, and thus a shortest path
                    shorthestPaths.add(currentPath);
                }
            }

            // no shortest paths found in this
            // iteration cycle, so we build up new
            // candidates (examined in next iteration cycle)
            // from current candidates and their parents
            if (shorthestPaths.isEmpty()) {
                final List newCandidates = new ArrayList();
                for (Iterator iterator = candidatePaths.iterator(); iterator.hasNext();) {
                    final List currentPath = (List) iterator.next();
                    final NodeMatcher current = (NodeMatcher) currentPath.get(0);
                    //not checking again as the value has already been checked above
                    final List parents = (List) reverseMap.get(current);

                    // the new candidates are current path
                    // prepended by all possible parents
                    for (Iterator iterator1 = parents.iterator(); iterator1.hasNext();) {
                        final Object m = iterator1.next();
                        if (!(m instanceof NodeMatcher)) {
                            throw new IllegalArgumentException("The given map was expected to only "
                                + "contain list that have elements of type NodeMatcher.");
                        }
                        final NodeMatcher parent = (NodeMatcher) m;
                        // we need to check whether the current path already
                        // contains the parent candidate,
                        // In that case we do not add this combination
                        // to the candidates list as it would be a cycle
                        if (!currentPath.contains(parent)) {
                            final List pathToParent = new ArrayList();
                            pathToParent.add(parent);
                            pathToParent.addAll(currentPath);
                            newCandidates.add(pathToParent);
                        }
                    }
                }
                candidatePaths = newCandidates;
            }
        }
        return shorthestPaths;
    }

    /**
     * This method selects a path from the given List of paths. It doe select the path that comes first
     * lexicographically by comparing the node's titles.
     *
     * @param paths the non-empty List of paths to select the 'first' from
     *
     * @return the selected path
     *
     * @throws IllegalArgumentException in case the given list is <tt>null</tt> or empty or does contain something
     *                                  different than {@link java.util.List}s containing {@link NodeMatcher} instances
     */
    static List getFirstPath(final List paths) {
        if (paths == null) {
            throw new IllegalArgumentException("The parameter named [paths] was null.");
        }
        if (paths.isEmpty()) {
            throw new IllegalArgumentException("The parameter named [shortestPaths] was an empty List.");
        }
        Collections.sort(paths, NODE_MATCHER_LIST_COMPARATOR);

        return (List) paths.get(0);
    }

    /**
     * This class implements a comparator that can be used to compare Lists that contain NodeMatcher instances.
     */
    private static class NodeMatcherListComparator implements Comparator {
        /**
         * Compares its two arguments for order.  Returns a negative integer, zero, or a positive integer as the first
         * argument is less than, equal to, or greater than the second.<p>
         * <p/>
         * The implementor must ensure that <tt>sgn(compare(x, y)) == -sgn(compare(y, x))</tt> for all <tt>x</tt> and
         * <tt>y</tt>.  (This implies that <tt>compare(x, y)</tt> must throw an exception if and only if <tt>compare(y,
         * x)</tt> throws an exception.)<p>
         * <p/>
         * The implementor must also ensure that the relation is transitive: <tt>((compare(x, y)&gt;0) &amp;&amp;
         * (compare(y, z)&gt;0))</tt> implies <tt>compare(x, z)&gt;0</tt>.<p>
         * <p/>
         * Finally, the implementer must ensure that <tt>compare(x, y)==0</tt> implies that <tt>sgn(compare(x,
         * z))==sgn(compare(y, z))</tt> for all <tt>z</tt>.<p>
         * <p/>
         * It is generally the case, but <i>not</i> strictly required that <tt>(compare(x, y)==0) == (x.equals(y))</tt>.
         * Generally speaking, any comparator that violates this condition should clearly indicate this fact.  The
         * recommended language is "Note: this comparator imposes orderings that are inconsistent with equals."
         *
         * @param o1 the first object to be compared.
         * @param o2 the second object to be compared.
         *
         * @return a negative integer, zero, or a positive integer as the first argument is less than, equal to, or
         *         greater than the second.
         *
         * @throws IllegalArgumentException if the arguments' types prevent them from being compared by this
         *                                  Comparator.
         */
        public int compare(final Object o1, final Object o2) {
            if (!(o1 instanceof List) || !(o2 instanceof List)) {
                throw new IllegalArgumentException("Expected the arguments to be of type List");
            }
            final List list1 = (List) o1;
            final List list2 = (List) o2;
            if (list1.size() != list2.size() || list1.isEmpty() || list2.isEmpty()) {
                throw new IllegalArgumentException("Expected to get two non-empty lists of equal size.");
            }
            final Iterator iterator1 = list1.iterator();
            final Iterator iterator2 = list2.iterator();
            while (iterator1.hasNext()) {
                final Object n1 = iterator1.next();
                final Object n2 = iterator2.next();
                if (!(n1 instanceof NodeMatcher) || !(n2 instanceof NodeMatcher)) {
                    throw new IllegalArgumentException(
                        "Expected the lists to contain elements of type NodeMatcher only.");
                }

                final int comparisonvalue = ((NodeMatcher) n1).getTitle().compareTo(((NodeMatcher) n2).getTitle());
                if (comparisonvalue != 0) {
                    // if we have found a
                    // difference, we return the comparison result,
                    // else we do the next iteration (nodes were
                    // equal, compare the next nodes in the paths)
                    return comparisonvalue;
                }
            }
            // when we have come here, the paths were completely equal,
            // as no difference has been found when iterating
            // the paths until end
            return 0;
        }
    }
}
