/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery;

import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbNode;

import java.io.Serializable;


/**
 * This class is a typical pair class that holds a paired association of a node matcher to the matching node. The {@link
 * NodeMatcher} describes the node that matched the above node and describes the default node information. The node
 * describes the actual node that matched the nodeMatcher and contains the contextual specific information.
 * <p/>
 * <b>Thread Safety: </b> This class is thread safe by being immutable.
 *
 * @author Pops, TCSDEVELOPER
 * @version 2.0
 * @since 2.0
 */
class NodePair implements Serializable {

    /**
     * This field represents the actual node that holds the contextual information (query string, etc) that matches the
     * node matcher.  This variable is set by the constructor, is immutable and can never be <tt>null</tt>,  The node
     * will be referenced in the {@link #getNode()} method.
     */
    private final BreadCrumbNode node;

    /**
     * This field represents the node matcher that holds the matching information and the default title.  This variable
     * is set by the constructor, is immutable and can be <tt>null</tt> (if there is no associated nodeMatcher).  The
     * path node will be referenced in the {@link #getNodeMatcher()} and {@link #getNode()} method.
     */
    private final NodeMatcher nodeMatcher;

    /**
     * Constructs the path pair from the given node and matcher.
     *
     * @param node        The non-<tt>null</tt> bread crumb node that matches the path node
     * @param nodeMatcher The possibly <tt>null</tt> node matcher
     *
     * @throws IllegalArgumentException if node is <tt>null</tt>
     */
    NodePair(final BreadCrumbNode node, final NodeMatcher nodeMatcher) {
        if (node == null) {
            throw new IllegalArgumentException("The parameter named [node] was null.");
        }

        this.nodeMatcher = nodeMatcher;
        if (node.getTitle() != null || nodeMatcher == null) {
            this.node = node;
        } else {
            this.node = new BreadCrumbNode(node.getUrl(), nodeMatcher.getTitle());
        }
    }

    /**
     * This method will simply return the node if the node's title is non-<tt>null</tt>. If the node's title is
     * <tt>null</tt>, a new node will be created with the title from the node matcher (if there is one).
     *
     * @return A non-<tt>null</tt> BreadCrumbNode
     */
    public BreadCrumbNode getNode() {
        return node;
    }

    /**
     * This method returns the node matcher.
     *
     * @return A possibly <tt>null</tt> node matcher
     */
    public NodeMatcher getNodeMatcher() {
        return nodeMatcher;
    }
}
