/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery.nodematchers;

import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbException;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbNode;
import com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery.AbstractNodeMatcher;

import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;


/**
 * This implementation of a {@link com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery.NodeMatcher} will provide node
 * matching services given a specific regex pattern. This class will return true when the passed node's url matches the
 * specified pattern.
 * <p/>
 * <b>Thread Safety: </b> This class is thread safe by being immutable.
 *
 * @author Pops, TCSDEVELOPER
 * @version 2.0
 * @since 2.0
 */
public class RegexNodeMatcher extends AbstractNodeMatcher {

    /**
     * This field represents the pattern that will be used to determine if the node matches.  This variable is set by
     * the constructor, is immutable and can never be <tt>null</tt> or an empty (trim'd) string,  The pattern will be
     * returned in the {@link #getNodePattern()} getter method.
     */
    private final String nodePattern;

    /**
     * This field represents the compiled version of the nodePattern.  This variable is set by the constructor, is
     * immutable and can never be <tt>null</tt>,  The pattern will be used in the {@link
     * #matches(BreadCrumbNode)}method.
     */
    private final Pattern pattern;

    /**
     * Constructs the RegexNodeMatcher from the given title, url and pattern.
     *
     * @param title       A non-<tt>null</tt>, non-empty (trim'd) string describing the default title of the node
     * @param url         A non-<tt>null</tt>, non-empty (trim'd) string representing the url to us
     * @param nodePattern A non-<tt>null</tt>, non-empty (trim'd) string representing the node pattern to apply
     *
     * @throws IllegalArgumentException if any argument is a <tt>null</tt> or an empty (trim'd) string
     * @throws BreadCrumbException      in case the given nodePattern does not compile to a regular expression
     */
    public RegexNodeMatcher(final String title, final String url, final String nodePattern) throws BreadCrumbException {
        super(title, url);
        if (nodePattern == null) {
            throw new IllegalArgumentException("The parameter named [nodePattern] was null.");
        }

        if (nodePattern.trim().length() == 0) {
            throw new IllegalArgumentException("The parameter named [nodePattern] was an empty String.");
        }
        this.nodePattern = nodePattern;
        try {
            pattern = Pattern.compile(nodePattern);
        } catch (PatternSyntaxException e) {
            throw new BreadCrumbException(
                "The given pattern [" + nodePattern + "] did not compile to a regular expression.", e);
        }
    }

    /**
     * This method checks whether the node's url matches this matcher's pattern.
     *
     * @param node A non-<tt>null</tt> node to test
     *
     * @return <tt>true</tt> if the node matches the regex pattern, <tt>false</tt> otherwise
     */
    public boolean matches(final BreadCrumbNode node) {
        if (node == null) {
            throw new IllegalArgumentException("The parameter named [node] was null.");
        }

        return pattern.matcher(node.getUrl()).matches();
    }

    /**
     * This method returns node pattern string.
     *
     * @return A non-<tt>null</tt>, non-empty (trim'd) string representing the regex pattern used to match against
     *         nodes
     */
    public String getNodePattern() {
        return nodePattern;
    }

    /**
     * This method checks whether the given object equals this node matcher.
     *
     * @param o A possibly <tt>null</tt> object to test for equality
     *
     * @return <tt>true</tt> if the object is of type RegexNodeMatcher with a matching nodePattern/title/url,
     *         <tt>false</tt> otherwise
     */
    public boolean equals(final Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }

        final RegexNodeMatcher that = (RegexNodeMatcher) o;

        return nodePattern.equals(that.nodePattern)
            && getTitle().equals(that.getTitle())
            && getUrl().equals(that.getUrl());

    }

    /**
     * This method calculates the hash code for this object.
     *
     * @return The hashcode of the object
     */
    public int hashCode() {
        return nodePattern.hashCode() + 29 ^ getTitle().hashCode() + 29 ^ getUrl().hashCode();
    }
}
